package com.gic.haoban.manage.service.service.impl;

import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.enums.FriendSettingTypeEnum;
import com.gic.haoban.manage.api.enums.WelcomeMediaTypeEnum;
import com.gic.haoban.manage.api.enums.WelcomeSuitTypeEnum;
import com.gic.haoban.manage.service.dao.mapper.*;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.pojo.bo.welcome.*;
import com.gic.haoban.manage.service.service.WelcomeService;
import com.gic.haoban.manage.service.service.WelcomeSuitRangService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * desc：欢迎语接口服务实现
 *
 * @author: YongEn
 * @date: 2022/2/9
 **/
@Service
public class WelcomeServiceImpl implements WelcomeService {

    private static final Logger logger = LoggerFactory.getLogger(WelcomeServiceImpl.class);

    @Autowired
    private TabHaobanWelcomeMapper welcomeMapper;
    @Autowired
    private TabHaobanWelcomeMediaMapper welcomeMediaMapper;
    @Autowired
    private TabHaobanFriendSettingMapper friendSettingMapper;
    @Autowired
    private WelcomeSuitRangService welcomeSuitRangService;
    @Autowired
    private TabHaobanWelcomeSuitRangMapper welcomeSuitRangMapper;
    @Autowired
    private StaffDepartmentRelatedMapper staffDepartmentRelatedMapper;
    @Autowired
    private TabMiniprogramSettingMapper miniprogramSettingMapper;

    private static final int WELCOME_LIMIT = 50;
    private static final int DEPARTMENT_NUM_LIMIT = 10000;
    private static final int STAFF_NUM_LIMIT = 2000;

    @Override
    public Boolean saveWelcome(SaveWelcomeBO bo) {
        // 校验参数和阈值
        String checkString = saveWelcomeCheck(bo);
        if (StringUtils.isNotEmpty(checkString)) {
            throw new RuntimeException(checkString);
        }
        // 校验新增或更新
        TabWelcome welcome = checkAddOrUpdate(bo);
        if (Objects.isNull(welcome)) {
            // 新增欢迎语
            TabWelcome addWelcome = addWelcome(bo);
            // 新增欢迎语媒体
            addWelcomeMedia(addWelcome.getWelcomeId(), bo);
            // 新增适用部门
            welcomeSuitRangService.addSuitDepartment(addWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitDepartmentAddList());
            // 新增适用员工
            welcomeSuitRangService.addSuitStaff(addWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitStaffAddList());
            return Boolean.TRUE;
        }
        // 更新欢迎语
        TabWelcome updateWelcome = updateWelcome(welcome, bo);
        // 删除欢迎语所有媒体
        welcomeMediaMapper.removeByWelcomeId(updateWelcome.getWelcomeId());
        // 新增欢迎语媒体
        addWelcomeMedia(updateWelcome.getWelcomeId(), bo);
        // 新增适用部门
        welcomeSuitRangService.addSuitDepartment(updateWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitDepartmentAddList());
        // 新增适用员工
        welcomeSuitRangService.addSuitStaff(updateWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitStaffAddList());
        // 删除适用部门
        welcomeSuitRangService.deleteSuitDepartment(updateWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitDepartmentDeleteList());
        // 删除适用员工
        welcomeSuitRangService.deleteSuitStaff(updateWelcome.getWelcomeId(), bo.getWxEnterpriseId(), bo.getSuitStaffDeleteList());
        return Boolean.TRUE;
    }

    private String saveWelcomeCheck(SaveWelcomeBO bo) {
        // 校验欢迎语数量、适用部门、适用员工数量
        long count = welcomeMapper.countByWxEnterpriseId(bo.getWxEnterpriseId());
        if (count >= WELCOME_LIMIT) {
            return "欢迎语条目超过限制";
        }
        List<WelcomeSuitDepartmentBO> suitDepartmentList = bo.getSuitDepartmentAddList();
        if (CollectionUtils.isNotEmpty(suitDepartmentList)) {
            if (suitDepartmentList.size() > DEPARTMENT_NUM_LIMIT) {
                return "适用部门数量超过限制";
            }
        }
        List<WelcomeSuitStaffBO> suitStaffList = bo.getSuitStaffAddList();
        if (CollectionUtils.isNotEmpty(suitStaffList)) {
            if (suitStaffList.size() > STAFF_NUM_LIMIT) {
                return "适用员工数量超过限制";
            }
        }
        return "";
    }

    private TabWelcome checkAddOrUpdate(SaveWelcomeBO bo) {
        String welcomeId = bo.getWelcomeId();
        if (StringUtils.isEmpty(welcomeId)) {
            return null;
        }
        return welcomeMapper.selectByPrimaryKey(bo.getWelcomeId());
    }

    private TabWelcome addWelcome(SaveWelcomeBO bo) {
        TabWelcome welcome = new TabWelcome();
        welcome.setWelcomeId(StringUtil.randomUUID());
        welcome.setWxEnterpriseId(bo.getWxEnterpriseId());
        welcome.setTitle(bo.getTitle());
        welcome.setWelcomeContent(bo.getWelcomeContent());
        welcome.setOpenFlag(1);
        welcome.setStatusFlag(1);
        welcome.setCreateTime(new Date());
        welcome.setUpdateTime(new Date());
        welcome.setOperatorId(bo.getOperatorId());
        welcome.setOperatorName(bo.getOperatorName());
        // 通过外部新增的统一走非默认
        welcome.setDefaultWelcomeFlag(0);
        welcomeMapper.insert(welcome);
        return welcome;
    }

    private TabWelcome updateWelcome(TabWelcome welcome, SaveWelcomeBO bo) {
        welcome.setTitle(bo.getTitle());
        welcome.setWelcomeContent(bo.getWelcomeContent());
        welcome.setUpdateTime(new Date());
        welcome.setOperatorId(bo.getOperatorId());
        welcome.setOperatorName(bo.getOperatorName());
        welcomeMapper.updateByPrimaryKeySelective(welcome);
        return welcome;
    }

    private void addWelcomeMedia(String welComeId, SaveWelcomeBO bo) {
        if (CollectionUtils.isEmpty(bo.getWelcomeMediaList())) {
            return;
        }
        List<TabHaobanWelcomeMedia> mediaList = bo.getWelcomeMediaList().stream().map(one -> {
            TabHaobanWelcomeMedia media = new TabHaobanWelcomeMedia();
            media.setWelcomeMediaId(UniqueIdUtils.uniqueLong());
            media.setWxEnterpriseId(bo.getWxEnterpriseId());
            media.setWelcomeId(welComeId);
            media.setMediaTitle(one.getMediaTitle());
            media.setMediaType(one.getMediaType());
            media.setMediaUrl(one.getMediaUrl());
            media.setMediaSize(one.getMediaSize());
            media.setCreateTime(new Date());
            media.setDeleteFlag(0);
            return media;
        }).collect(Collectors.toList());
        // 批量新增
        mediaList.forEach(one -> welcomeMediaMapper.insert(one));
    }

    @Override
    public Boolean removeWelcome(String welcomeId) {
        TabWelcome tabWelcome = new TabWelcome();
        tabWelcome.setWelcomeId(welcomeId);
        tabWelcome.setStatusFlag(0);
        int i = welcomeMapper.updateByPrimaryKeySelective(tabWelcome);
        return i > 0;
    }

    @Override
    public List<WelcomeListBO> listWelcome(String wxEnterpriseId, String title) {
        // 查询所有类型 默认 非默认
        List<TabWelcome> welcomeList = welcomeMapper.listByWxEnterpriseId(wxEnterpriseId, title, null);
        if (CollectionUtils.isEmpty(welcomeList)) {
            return Collections.emptyList();
        }
        // 组装数据, 转换数据
        return welcomeList.stream().map(one -> {
            WelcomeListBO bo = new WelcomeListBO();
            bo.setWelcomeId(one.getWelcomeId());
            bo.setWxEnterpriseId(one.getWxEnterpriseId());
            bo.setTitle(one.getTitle());
            bo.setWelcomeContent(one.getWelcomeContent());
            bo.setOpenFlag(one.getOpenFlag());
            bo.setUpdateTime(one.getUpdateTime());
            bo.setOperatorId(one.getOperatorId());
            bo.setOperatorName(one.getOperatorName());
            bo.setDefaultWelcomeFlag(one.getDefaultWelcomeFlag());
            // 媒体列表
            bo.setWelcomeMediaList(
                    convertWelcomeMedia(welcomeMediaMapper.listByWelcomeId(one.getWelcomeId()))
            );
            // 适用部门列表
            TabHaobanWelcomeSuitRang deptRangQuery = new TabHaobanWelcomeSuitRang();
            deptRangQuery.setWxEnterpriseId(one.getWxEnterpriseId());
            deptRangQuery.setWelcomeId(one.getWelcomeId());
            deptRangQuery.setSuitType(WelcomeSuitTypeEnum.DEPARTMENT.getCode());
            deptRangQuery.setSelectType(1);
            deptRangQuery.setDeleteFlag(0);
            bo.setSuitDepartmentList(
                    convertSuitDepartment(welcomeSuitRangMapper.queryAll(deptRangQuery))
            );
            // 适用员工列表
            TabHaobanWelcomeSuitRang staffRangQuery = new TabHaobanWelcomeSuitRang();
            staffRangQuery.setWxEnterpriseId(one.getWxEnterpriseId());
            staffRangQuery.setWelcomeId(one.getWelcomeId());
            staffRangQuery.setSuitType(WelcomeSuitTypeEnum.STAFF.getCode());
            staffRangQuery.setSelectType(1);
            staffRangQuery.setDeleteFlag(0);
            bo.setSuitStaffList(
                    convertSuitStaff(welcomeSuitRangMapper.queryAll(staffRangQuery))
            );
            return bo;
        }).collect(Collectors.toList());
    }

    @Override
    public WelcomeDetailBO getWelcome(String welcomeId, String enterpriseId) {
        TabWelcome welcome = welcomeMapper.selectByPrimaryKey(welcomeId);
        if (Objects.isNull(welcome)) {
            throw new RuntimeException("欢迎语数据不存在");
        }
        // 封装数据
        WelcomeDetailBO welcomeDetail = new WelcomeDetailBO();
        welcomeDetail.setWelcomeId(welcome.getWelcomeId());
        welcomeDetail.setWxEnterpriseId(welcome.getWxEnterpriseId());
        welcomeDetail.setTitle(welcome.getTitle());
        welcomeDetail.setWelcomeContent(welcome.getWelcomeContent());
        welcomeDetail.setOpenFlag(welcome.getOpenFlag());
        welcomeDetail.setDefaultWelcomeFlag(welcome.getDefaultWelcomeFlag());
        // 媒体列表
        welcomeDetail.setWelcomeMediaList(
                convertWelcomeMedia(welcomeMediaMapper.listByWelcomeId(welcome.getWelcomeId()))
        );
        // 为媒体列表添加小程序
        assemblyMiniProgram(welcome, welcomeDetail, enterpriseId);

        // 适用部门列表
        TabHaobanWelcomeSuitRang deptRangQuery = new TabHaobanWelcomeSuitRang();
        deptRangQuery.setWxEnterpriseId(welcome.getWxEnterpriseId());
        deptRangQuery.setWelcomeId(welcome.getWelcomeId());
        deptRangQuery.setSuitType(WelcomeSuitTypeEnum.DEPARTMENT.getCode());
        deptRangQuery.setDeleteFlag(0);
        welcomeDetail.setSuitDepartmentList(
                convertSuitDepartment(welcomeSuitRangMapper.queryAll(deptRangQuery))
        );
        // 适用员工列表
        TabHaobanWelcomeSuitRang staffRangQuery = new TabHaobanWelcomeSuitRang();
        staffRangQuery.setWxEnterpriseId(welcome.getWxEnterpriseId());
        staffRangQuery.setWelcomeId(welcome.getWelcomeId());
        staffRangQuery.setSuitType(WelcomeSuitTypeEnum.STAFF.getCode());
        staffRangQuery.setDeleteFlag(0);
        welcomeDetail.setSuitStaffList(
                convertSuitStaff(welcomeSuitRangMapper.queryAll(staffRangQuery))
        );
        return welcomeDetail;
    }

    private void assemblyMiniProgram(TabWelcome welcome, WelcomeDetailBO welcomeDetail, String enterpriseId) {
        TabHaobanFriendSetting friendSetting = friendSettingMapper.getByWxEnterpriseId(welcome.getWxEnterpriseId());
        if (Objects.nonNull(friendSetting) && !FriendSettingTypeEnum.TEXT_AND_MINI_PROGRAM.getCode().equals(friendSetting.getFriendSettingType())) {
            return;
        }
        TabMiniprogramSetting matchMiniProgram = null;
        if (StringUtils.isNotEmpty(enterpriseId)) {
             matchMiniProgram = miniprogramSettingMapper.getMiniprogramSetting(welcome.getWxEnterpriseId(), enterpriseId);
        }
        if (Objects.isNull(matchMiniProgram)) {
            List<TabMiniprogramSetting> miniProgramSettings = miniprogramSettingMapper.listMiniprogramSetting(welcome.getWxEnterpriseId());
            if (CollectionUtils.isNotEmpty(miniProgramSettings)) {
                matchMiniProgram = miniProgramSettings.get(0);
            }
        }
        if (Objects.isNull(matchMiniProgram)) {
            logger.info("无小程序配置, {}", welcome.getWxEnterpriseId());
            return;
        }
        WelcomeMediaBO welcomeMediaBO = new WelcomeMediaBO();
        welcomeMediaBO.setWxEnterpriseId(welcome.getWxEnterpriseId());
        welcomeMediaBO.setMediaTitle(matchMiniProgram.getTitle());
        welcomeMediaBO.setMediaType(WelcomeMediaTypeEnum.PROGRAM.getCode());
        welcomeMediaBO.setMiniProgramName(matchMiniProgram.getMiniprogramName());
        welcomeMediaBO.setMediaUrl(matchMiniProgram.getImageUrl());
        if (CollectionUtils.isEmpty(welcomeDetail.getWelcomeMediaList())) {
            welcomeDetail.setWelcomeMediaList(new ArrayList<>());
        }
        welcomeDetail.getWelcomeMediaList().add(welcomeMediaBO);
    }

    @Override
    public Boolean saveFriendSetting(SaveFriendSettingBO bo) {
        if (Objects.isNull(bo)) {
            throw new RuntimeException("参数为空");
        }
        TabHaobanFriendSetting friendSetting = null;
        if (Objects.nonNull(bo.getFriendSettingId())) {
            friendSetting = friendSettingMapper.queryById(bo.getFriendSettingId());
        }
        if (Objects.isNull(friendSetting)) {
            // 新增
            friendSetting = new TabHaobanFriendSetting();
            friendSetting.setFriendSettingId(UniqueIdUtils.uniqueLong());
            friendSetting.setWxEnterpriseId(bo.getWxEnterpriseId());
            friendSetting.setFriendSettingType(bo.getFriendSettingType());
            friendSetting.setUpdateId(bo.getUpdateId());
            friendSetting.setUpdateName(bo.getUpdateName());
            friendSetting.setCreateTime(new Date());
            friendSetting.setDeleteFlag(0);
            friendSettingMapper.insert(friendSetting);
        } else {
            // 更新
            friendSetting.setFriendSettingType(bo.getFriendSettingType());
            friendSetting.setUpdateId(bo.getUpdateId());
            friendSetting.setUpdateName(bo.getUpdateName());
            friendSetting.setUpdateTime(new Date());
            friendSettingMapper.update(friendSetting);
        }
        return Boolean.TRUE;
    }

    @Override
    public FriendSettingDetailBO getFriendSetting(String wxEnterpriseId) {
        TabHaobanFriendSetting friendSetting = friendSettingMapper.getByWxEnterpriseId(wxEnterpriseId);
        FriendSettingDetailBO friendSettingDetail = new FriendSettingDetailBO();
        if (Objects.isNull(friendSetting)) {
            // 返回默认
            friendSettingDetail.setFriendSettingType(FriendSettingTypeEnum.TEXT_AND_MINI_PROGRAM.getCode());
            friendSettingDetail.setWxEnterpriseId(wxEnterpriseId);
        } else {
            friendSettingDetail.setFriendSettingId(friendSetting.getFriendSettingId());
            friendSettingDetail.setWxEnterpriseId(friendSetting.getWxEnterpriseId());
            friendSettingDetail.setFriendSettingType(friendSetting.getFriendSettingType());
        }
        return friendSettingDetail;
    }

    @Override
    public WelcomeDetailBO getMatchWelcome(String staffId, String wxEnterpriseId) {
        if (StringUtils.isEmpty(staffId) || StringUtils.isEmpty(wxEnterpriseId)) {
            return null;
        }
        // 查询返回默认欢迎语
        List<TabWelcome> defaultWelcomeList = welcomeMapper.listByWxEnterpriseId(wxEnterpriseId, null, 1);
        if (CollectionUtils.isEmpty(defaultWelcomeList)) {
            logger.info("当前企业{}不存在默认欢迎语", wxEnterpriseId);
            return null;
        }
        // 匹配欢迎语, 查询列表, 查询非默认欢迎语
        List<TabWelcome> welcomeList = welcomeMapper.listByWxEnterpriseId(wxEnterpriseId, null, 0);
        if (CollectionUtils.isEmpty(welcomeList)) {
            logger.info("当前企业{}不存在配置欢迎语, 返回默认欢迎语", wxEnterpriseId);
            return convertWelcomeToBo(defaultWelcomeList.get(0));
        }

        // 匹配员工 -- 暂时不做

        // 匹配员工部门,查询员工部门关联
        List<TabHaobanStaffDepartmentRelated> departmentRelatedList = staffDepartmentRelatedMapper.listByWxEnterpriseIdAndStaffId(wxEnterpriseId, staffId);
        if (CollectionUtils.isEmpty(departmentRelatedList)) {
            logger.info("员工{}, 未设置部门", staffId);
            return null;
        }
        List<TabHaobanStaffDepartmentRelated> filterStaffDepartmentList = departmentRelatedList.stream()
                .filter(one -> StringUtils.isNotEmpty(one.getDepartmentId()))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(filterStaffDepartmentList)) {
            logger.info("员工{}, 部门过滤为空", staffId);
            return null;
        }
        // 获取部门列表
        List<String> departmentIdList = filterStaffDepartmentList.stream()
                .map(TabHaobanStaffDepartmentRelated::getDepartmentId).collect(Collectors.toList());
        // 查询欢迎语适用范围
        List<TabHaobanWelcomeSuitRang> welcomeSuitRangList = welcomeSuitRangMapper.listByWxEnterpriseIdAndDeptList(wxEnterpriseId, departmentIdList);
        if (CollectionUtils.isEmpty(welcomeSuitRangList)) {
            logger.info("当前员工所在部门不存在匹配欢迎语, 返回默认欢迎语");
            return convertWelcomeToBo(defaultWelcomeList.get(0));
        }
        // welcomeList 已经经过时间排序, 顺序去对适用欢迎语匹配, 第一条即是匹配上的欢迎语
        WelcomeDetailBO matchWelcome = null;
        Set<String> welcomeIdSet = welcomeSuitRangList.stream().map(TabHaobanWelcomeSuitRang::getWelcomeId).collect(Collectors.toSet());
        for (TabWelcome welcome : welcomeList) {
            if (welcomeIdSet.contains(welcome.getWelcomeId())) {
                matchWelcome = convertWelcomeToBo(welcome);
                break;
            }
        }
        if (Objects.isNull(matchWelcome)) {
            return convertWelcomeToBo(defaultWelcomeList.get(0));
        }
        return matchWelcome;
    }

    @Override
    public Boolean saveDefaultWelcome(String wxEnterpriseId) {
        List<TabWelcome> welcomeList = welcomeMapper.listByWxEnterpriseId(wxEnterpriseId, null, 1);
        if (CollectionUtils.isNotEmpty(welcomeList)) {
            throw new RuntimeException("已存在默认欢迎语");
        }
        // 构造默认欢迎语
        TabWelcome welcome = new TabWelcome();
        welcome.setWelcomeId(StringUtil.randomUUID());
        welcome.setWxEnterpriseId(wxEnterpriseId);
        welcome.setTitle("全员欢迎语");
        welcome.setWelcomeContent("你好,<微信昵称>长按识别图中二维码关注公众号并成为会员吧！");
        welcome.setWelcomeImg("");
        // 默认开启
        welcome.setOpenFlag(1);
        welcome.setStatusFlag(1);
        welcome.setCreateTime(new Date());
        welcome.setUpdateTime(new Date());
        welcome.setOperatorId("-1");
        welcome.setOperatorName("system");
        welcome.setDefaultWelcomeFlag(1);
        // 插入
        welcomeMapper.insert(welcome);
        return Boolean.TRUE;
    }

    private WelcomeDetailBO convertWelcomeToBo(TabWelcome welcome) {
        WelcomeDetailBO matchWelcome = new WelcomeDetailBO();
        matchWelcome.setWelcomeId(welcome.getWelcomeId());
        matchWelcome.setWxEnterpriseId(welcome.getWxEnterpriseId());
        matchWelcome.setTitle(welcome.getTitle());
        matchWelcome.setWelcomeContent(welcome.getWelcomeContent());
        matchWelcome.setOpenFlag(welcome.getOpenFlag());
        matchWelcome.setDefaultWelcomeFlag(welcome.getDefaultWelcomeFlag());
        // 媒体列表
        matchWelcome.setWelcomeMediaList(
                convertWelcomeMedia(welcomeMediaMapper.listByWelcomeId(welcome.getWelcomeId()))
        );
        return matchWelcome;
    }

    private List<WelcomeMediaBO> convertWelcomeMedia(List<TabHaobanWelcomeMedia> welcomeMediaList) {
        if (CollectionUtils.isEmpty(welcomeMediaList)) {
            return Collections.emptyList();
        }
        return welcomeMediaList.stream().map(one -> {
            WelcomeMediaBO bo = new WelcomeMediaBO();
            bo.setWelcomeMediaId(one.getWelcomeMediaId());
            bo.setWelcomeId(one.getWelcomeId());
            bo.setWxEnterpriseId(one.getWxEnterpriseId());
            bo.setMediaTitle(one.getMediaTitle());
            bo.setMediaType(one.getMediaType());
            bo.setMediaUrl(one.getMediaUrl());
            return bo;
        }).collect(Collectors.toList());
    }

    private List<WelcomeSuitDepartmentBO> convertSuitDepartment(List<TabHaobanWelcomeSuitRang> deptSuitRangList) {
        if (CollectionUtils.isEmpty(deptSuitRangList)) {
            return Collections.emptyList();
        }
        return deptSuitRangList.stream().map(one -> {
            WelcomeSuitDepartmentBO bo = new WelcomeSuitDepartmentBO();
            bo.setDepartmentId(one.getSuitDepartmentId());
            bo.setDepartmentName(one.getSuitDepartmentName());
            bo.setSelectType(one.getSelectType());
            return bo;
        }).collect(Collectors.toList());
    }

    private List<WelcomeSuitStaffBO> convertSuitStaff(List<TabHaobanWelcomeSuitRang> staffSuitRangList) {
        if (CollectionUtils.isEmpty(staffSuitRangList)) {
            return Collections.emptyList();
        }
        return staffSuitRangList.stream().map(one -> {
            WelcomeSuitStaffBO bo = new WelcomeSuitStaffBO();
            bo.setStaffId(one.getSuitStaffId());
            bo.setStaffName(one.getSuitStaffName());
            bo.setBelongDepartmentId(one.getStaffBelongDepartmentId());
            bo.setBelongDepartmentName(one.getStaffBelongDepartmentName());
            return bo;
        }).collect(Collectors.toList());
    }
}
